unit main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls,  // Das sind die Units mit den zu untersuchenden Klassen
  LvlGraphCtrl;

type

  { TMainForm }

  TMainForm = class(TForm)
    ImageList1: TImageList;
    LvlGraphControl1: TLvlGraphControl;
    procedure FormCreate(Sender: TObject);
  private
    FRootClassName: String;
    FImages: TStrings;
    function AddClassNode(AClass: TClass; ARootClassName: String=''): TLvlGraphNode;
    function GetImageIndex(AClassName: String): Integer;

  public
    procedure AddControls;

  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

const
  ImagesDir = 'C:\lazarus-trunk_fpc304\images\components\';

{ TMainForm }

function TMainForm.AddClassNode(AClass: TClass; ARootClassName: String = ''): TLvlGraphNode;
var
  parentclass: TClass;
begin
  if ARootClassName <> '' then
    FRootClassName := ARootClassName;
  Result := LvlGraphControl1.Graph.GetNode(AClass.ClassName, true);
  Result.ImageIndex := GetImageIndex(AClass.ClassName);
  if AClass.ClassName = FRootClassName then
    exit;
  parentclass := AClass.ClassParent;
  AddClassNode(parentclass);
  LvlGraphControl1.Graph.GetEdge(parentclass.ClassName, AClass.ClassName, true);
end;

procedure TMainForm.AddControls;
begin
  // aus StdCtrls
  AddClassNode(TEdit);
  AddClassNode(TLabel);
  AddClassNode(TButton);

  // aus ComCtrls
  AddClassNode(TTreeView);

  // aus ExtCtrls
  AddClassNode(TImage);

  // Das könnte man auch im Objekt-Inspektor einstellen...
  with LvlGraphControl1 do begin
    Options := LvlGraphControl1.Options + [lgoStraightenGraph];
    Images := ImageList1;
    NodeStyle.Shape := lgnsNone;
    NodeStyle.GapTop := 16;
    NodeStyle.GapBottom := 4;
    NodeStyle.GapLeft := 10;
    NodeStyle.GapRight := 10;
    NodeStyle.CaptionPosition := lgncBottom;
    NodeStyle.CaptionScale := 1.0;
    EdgeStyle.Shape := lgesStraight;
    Limits.MaxLevelHeightAbs := MaxInt;
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  FRootClassName := 'TComponent';
  FImages := TStringList.Create;
  ImageList1.Width := 24;
  ImageList1.Height := 24;

  AddControls;
end;

function TMainForm.GetImageIndex(AClassName: String): Integer;
var
  png: TCustomBitmap;
  fn: String;
begin
  result := -1;
  fn := ImagesDir + AClassName + '.png';
  if not FileExists(fn) then
    fn := ImagesDir + 'default.png';
  if FileExists(fn) then
  begin
    Result := FImages.IndexOf(fn);
    if Result = - 1 then
    begin
      png := TPortableNetworkgraphic.Create;
      try
        png.LoadFromFile(fn);
        Result := ImageList1.Add(png, nil);
        FImages.Add(fn);
      finally
        png.Free;
      end;
    end;
  end;
end;

end.

